=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2010-10-13 16:44:48 +0000
+++ bzrlib/__init__.py	2010-11-05 15:57:40 +0000
@@ -52,7 +52,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 3, 0, 'dev', 3)
+version_info = (2, 3, 0, 'dev', 4)
 
 # API compatibility version
 api_minimum_version = (2, 3, 0)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-10-20 12:06:13 +0000
+++ bzrlib/builtins.py	2010-11-04 17:48:47 +0000
@@ -5894,7 +5894,7 @@
             location = "."
         branch = Branch.open_containing(location)[0]
         branch.bzrdir.destroy_branch()
-        
+
 
 class cmd_shelve(Command):
     __doc__ = """Temporarily set aside some changes from the current tree.
@@ -5951,9 +5951,9 @@
     _see_also = ['unshelve', 'configuration']
 
     def run(self, revision=None, all=False, file_list=None, message=None,
-            writer=None, list=False, destroy=False, directory=u'.'):
+            writer=None, list=False, destroy=False, directory=None):
         if list:
-            return self.run_for_list()
+            return self.run_for_list(directory=directory)
         from bzrlib.shelf_ui import Shelver
         if writer is None:
             writer = bzrlib.option.diff_writer_registry.get()
@@ -5967,8 +5967,10 @@
         except errors.UserAbort:
             return 0
 
-    def run_for_list(self):
-        tree = WorkingTree.open_containing('.')[0]
+    def run_for_list(self, directory=None):
+        if directory is None:
+            directory = u'.'
+        tree = WorkingTree.open_containing(directory)[0]
         self.add_cleanup(tree.lock_read().unlock)
         manager = tree.get_shelf_manager()
         shelves = manager.active_shelves()

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2010-07-16 15:35:43 +0000
+++ bzrlib/conflicts.py	2010-11-04 17:48:47 +0000
@@ -117,17 +117,19 @@
             ResolveActionOption(),
             ]
     _see_also = ['conflicts']
-    def run(self, file_list=None, all=False, action=None, directory=u'.'):
+    def run(self, file_list=None, all=False, action=None, directory=None):
         if all:
             if file_list:
                 raise errors.BzrCommandError("If --all is specified,"
                                              " no FILE may be provided")
+            if directory is None:
+                directory = u'.'
             tree = workingtree.WorkingTree.open_containing(directory)[0]
             if action is None:
                 action = 'done'
         else:
             tree, file_list = workingtree.WorkingTree.open_containing_paths(
-                file_list)
+                file_list, directory)
             if file_list is None:
                 if action is None:
                     # FIXME: There is a special case here related to the option

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2010-04-30 11:35:43 +0000
+++ bzrlib/delta.py	2010-10-20 14:38:53 +0000
@@ -61,6 +61,7 @@
         self.modified = []
         self.unchanged = []
         self.unversioned = []
+        self.missing = []
 
     def __eq__(self, other):
         if not isinstance(other, TreeDelta):
@@ -137,7 +138,7 @@
             else:
                 delta.removed.append((path[0], file_id, kind[0]))
         elif fully_present[0] is False:
-            continue
+            delta.missing.append((path[1], file_id, kind[1]))
         elif name[0] != name[1] or parent_id[0] != parent_id[1]:
             # If the name changes, or the parent_id changes, we have a rename
             # (if we move a parent, that doesn't count as a rename for the
@@ -160,6 +161,7 @@
     delta.removed.sort()
     delta.added.sort()
     delta.renamed.sort()
+    delta.missing.sort()
     # TODO: jam 20060529 These lists shouldn't need to be sorted
     #       since we added them in alphabetical order.
     delta.modified.sort()
@@ -202,7 +204,9 @@
                              'unchanged': ' ',
                              'created': 'N',
                              'modified': 'M',
-                             'deleted': 'D'}
+                             'deleted': 'D',
+                             'missing': '!',
+                             }
         self.versioned_map = {'added': '+', # versioned target
                               'unchanged': ' ', # versioned in both
                               'removed': '-', # versioned in source
@@ -325,6 +329,8 @@
         else:
             if content_change:
                 modified = "modified"
+            elif kind[0] is None:
+                modified = "missing"
             else:
                 modified = "unchanged"
             if kind[1] == "file":
@@ -417,6 +423,7 @@
 
     show_list(delta.removed, 'removed', 'D')
     show_list(delta.added, 'added', 'A')
+    show_list(delta.missing, 'missing', '!')
     extra_modified = []
     # Reorder delta.renamed tuples so that all lists share the same
     # order for their 3 first fields and that they also begin like

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-10-13 08:01:36 +0000
+++ bzrlib/errors.py	2010-11-05 14:07:45 +0000
@@ -723,6 +723,15 @@
                     self.bzrdir.open_repository()
                 except NoRepositoryPresent:
                     self.detail = ''
+                except Exception:
+                    # Just ignore unexpected errors.  Raising arbitrary errors
+                    # during str(err) can provoke strange bugs.  Concretely
+                    # Launchpad's codehosting managed to raise NotBranchError
+                    # here, and then get stuck in an infinite loop/recursion
+                    # trying to str() that error.  All this error really cares
+                    # about that there's no working repository there, and if
+                    # open_repository() fails, there probably isn't.
+                    self.detail = ''
                 else:
                     self.detail = ': location is a repository'
             else:

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2010-08-17 06:45:33 +0000
+++ bzrlib/mutabletree.py	2010-10-17 23:20:40 +0000
@@ -545,6 +545,13 @@
                         this_ie = None
                     else:
                         this_ie = inv[this_id]
+                        # Same as in _add_one below, if the inventory doesn't
+                        # think this is a directory, update the inventory
+                        if this_ie.kind != 'directory':
+                            this_ie = inventory.make_entry('directory',
+                                this_ie.name, this_ie.parent_id, this_id)
+                            del inv[this_id]
+                            inv.add(this_ie)
 
                 for subf in sorted(os.listdir(abspath)):
                     # here we could use TreeDirectory rather than

=== modified file 'bzrlib/rules.py'
--- bzrlib/rules.py	2010-02-02 06:30:43 +0000
+++ bzrlib/rules.py	2010-10-31 14:31:17 +0000
@@ -74,8 +74,7 @@
 
         :param inifile: the name of the file or a sequence of lines.
         """
-        options = {'encoding': 'utf-8'}
-        self._cfg = configobj.ConfigObj(inifile, options=options)
+        self._cfg = configobj.ConfigObj(inifile, encoding='utf-8')
         sections = self._cfg.keys()
         patterns = []
         self.pattern_to_section = {}

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2010-09-20 23:15:01 +0000
+++ bzrlib/shelf_ui.py	2010-11-04 17:48:47 +0000
@@ -154,7 +154,7 @@
 
     @classmethod
     def from_args(klass, diff_writer, revision=None, all=False, file_list=None,
-                  message=None, directory='.', destroy=False):
+                  message=None, directory=None, destroy=False):
         """Create a shelver from commandline arguments.
 
         The returned shelver wil have a work_tree that is locked and should
@@ -168,6 +168,10 @@
         :param destroy: Change the working tree without storing the shelved
             changes.
         """
+        if directory is None:
+            directory = u'.'
+        elif file_list:
+            file_list = [osutils.pathjoin(directory, f) for f in file_list]
         tree, path = workingtree.WorkingTree.open_containing(directory)
         # Ensure that tree is locked for the lifetime of target_tree, as
         # target tree may be reading from the same dirstate.

=== modified file 'bzrlib/tests/blackbox/test_conflicts.py'
--- bzrlib/tests/blackbox/test_conflicts.py	2010-05-28 14:15:28 +0000
+++ bzrlib/tests/blackbox/test_conflicts.py	2010-11-04 17:48:47 +0000
@@ -103,6 +103,11 @@
         self.assertEqual(True, conflicts.is_empty(),
                          "tree still contains conflicts: %r" % conflicts)
 
+    def test_resolve_via_directory(self):
+        """resolve when run from subdirectory should handle relative paths"""
+        self.build_tree(["a/subdir/"])
+        self.run_bzr("resolve -d a/subdir ../myfile")
+
     def test_auto_resolve(self):
         """Text conflicts can be resolved automatically"""
         tree = self.make_branch_and_tree('tree')

=== modified file 'bzrlib/tests/blackbox/test_shelve.py'
--- bzrlib/tests/blackbox/test_shelve.py	2010-10-07 07:51:54 +0000
+++ bzrlib/tests/blackbox/test_shelve.py	2010-11-04 17:48:47 +0000
@@ -41,6 +41,14 @@
         self.assertEqual('', err)
         self.assertEqual('  1: Foo\n', out)
 
+    def test_shelve_list_via_directory(self):
+        tree = self.make_branch_and_tree('tree')
+        creator = self.make_creator(tree)
+        shelf_id = tree.get_shelf_manager().shelve_changes(creator, 'Foo')
+        out, err = self.run_bzr('shelve -d tree --list', retcode=1)
+        self.assertEqual('', err)
+        self.assertEqual('  1: Foo\n', out)
+
     def test_shelve_no_message(self):
         tree = self.make_branch_and_tree('.')
         creator = self.make_creator(tree)
@@ -125,6 +133,12 @@
         os.chdir('tree/dir')
         self.run_bzr('shelve --all ../file')
 
+    def test_shelve_via_directory(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file', 'tree/dir/'])
+        tree.add('file')
+        self.run_bzr('shelve -d tree/dir --all ../file')
+
 
 class TestShelveUnshelve(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2010-10-18 21:34:05 +0000
+++ bzrlib/tests/blackbox/test_status.py	2010-10-29 17:26:03 +0000
@@ -211,12 +211,18 @@
         wt = self.make_branch_and_tree('.')
         b = wt.branch
 
-        self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
+        self.build_tree(['directory/','directory/hello.c',
+                         'bye.c','test.c','dir2/',
+                         'missing.c'])
         wt.add('directory')
         wt.add('test.c')
         wt.commit('testing')
+        wt.add('missing.c')
+        unlink('missing.c')
 
         self.assertStatus([
+                'missing:\n',
+                '  missing.c\n',
                 'unknown:\n',
                 '  bye.c\n',
                 '  dir2/\n',
@@ -227,6 +233,7 @@
         self.assertStatus([
                 '?   bye.c\n',
                 '?   dir2/\n',
+                '+!  missing.c\n',
                 '?   directory/hello.c\n'
                 ],
                 wt, short=True)
@@ -269,6 +276,20 @@
         tof.seek(0)
         self.assertEquals(tof.readlines(), ['+N  test.c\n'])
 
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['missing.c'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['missing:\n',
+                           '  missing.c\n'])
+
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['missing.c'], to_file=tof,
+                         short=True)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['+!  missing.c\n'])
+
     def test_specific_files_conflicts(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir2/'])

=== modified file 'bzrlib/tests/per_workingtree/__init__.py'
--- bzrlib/tests/per_workingtree/__init__.py	2010-09-06 14:36:02 +0000
+++ bzrlib/tests/per_workingtree/__init__.py	2010-10-26 13:53:31 +0000
@@ -62,7 +62,7 @@
     def make_branch_builder(self, relpath, format=None):
         if format is None:
             format = self.bzrdir_format
-        builder = branchbuilder.BranchBuilder(self.get_transport(),
+        builder = branchbuilder.BranchBuilder(self.get_transport(relpath),
                                               format=format)
         return builder
 

=== modified file 'bzrlib/tests/per_workingtree/test_smart_add.py'
--- bzrlib/tests/per_workingtree/test_smart_add.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_workingtree/test_smart_add.py	2010-11-02 22:23:40 +0000
@@ -17,6 +17,7 @@
 """Test that we can use smart_add on all Tree implementations."""
 
 from cStringIO import StringIO
+import os
 import sys
 
 from bzrlib import (
@@ -202,6 +203,37 @@
         self.assertEqual(['', 'dir', 'dir/subdir', 'dir/subdir/foo'],
             [path for path, ie in tree.iter_entries_by_dir()])
 
+    def test_add_dir_bug_251864(self):
+        """Added file turning into a dir should be detected on add dir
+
+        Similar to bug 205636 but with automatic adding of directory contents.
+        """
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["dir"]) # whoops, make a file called dir
+        tree.smart_add(["dir"])
+        os.remove("dir")
+        self.build_tree(["dir/", "dir/file"])
+        tree.smart_add(["dir"])
+        tree.commit("Add dir contents")
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([(u"dir", "directory"), (u"dir/file", "file")],
+            [(t[0], t[2]) for t in tree.list_files()])
+        self.assertFalse(list(tree.iter_changes(tree.basis_tree())))
+
+    def test_add_subdir_file_bug_205636(self):
+        """Added file turning into a dir should be detected on add dir/file"""
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["dir"]) # whoops, make a file called dir
+        tree.smart_add(["dir"])
+        os.remove("dir")
+        self.build_tree(["dir/", "dir/file"])
+        tree.smart_add(["dir/file"])
+        tree.commit("Add file in dir")
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([(u"dir", "directory"), (u"dir/file", "file")],
+            [(t[0], t[2]) for t in tree.list_files()])
+        self.assertFalse(list(tree.iter_changes(tree.basis_tree())))
+
     def test_custom_ids(self):
         sio = StringIO()
         action = test_smart_add.AddCustomIDAction(to_file=sio,

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2010-10-26 13:53:31 +0000
@@ -45,6 +45,11 @@
 
 class TestWorkingTree(TestCaseWithWorkingTree):
 
+    def test_branch_builder(self):
+        # Just a smoke test that we get a branch at the specified relpath
+        builder = self.make_branch_builder('foobar')
+        br = branch.Branch.open('foobar')
+
     def test_list_files(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/', 'file'])

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2010-10-20 12:06:13 +0000
+++ bzrlib/tests/script.py	2010-10-28 00:06:59 +0000
@@ -213,7 +213,7 @@
         method = getattr(self, mname, None)
         if method is None:
             raise SyntaxError('Command not found "%s"' % (cmd[0],),
-                              None, 1, ' '.join(cmd))
+                              (None, 1, 1, ' '.join(cmd)))
         if input is None:
             str_input = ''
         else:

=== modified file 'bzrlib/tests/test_delta.py'
--- bzrlib/tests/test_delta.py	2010-10-08 05:36:17 +0000
+++ bzrlib/tests/test_delta.py	2010-10-20 14:38:53 +0000
@@ -122,6 +122,12 @@
             renamed=False, modified='created', exe_change=False,
             kind=(None, 'file'), unversioned_filter=lambda x:True)
 
+    def test_missing(self):
+        self.assertReport('+!  missing.c', file_id=None, path='missing.c',
+             old_path=None, versioned_change='added',
+             renamed=False, modified='missing', exe_change=False,
+             kind=(None, None))
+
     def test_view_filtering(self):
         # If a file in within the view, it should appear in the output
         expected_lines = [
@@ -279,24 +285,32 @@
                                   ('branch/f2', '2\n'),
                                   ('branch/f3', '3\n'),
                                   ('branch/f4', '4\n'),
+                                  ('branch/f5', '5\n'),
                                   ('branch/dir/',),
                                  ])
         wt.add(['f1', 'f2', 'f3', 'f4', 'dir'],
                ['f1-id', 'f2-id', 'f3-id', 'f4-id', 'dir-id'])
         wt.commit('commit one', rev_id='1')
 
+        # TODO add rename,removed,etc. here?
+        wt.add('f5')
+        os.unlink('branch/f5')
+
         long_status = """added:
   dir/
   f1
   f2
   f3
   f4
+missing:
+  f5
 """
         short_status = """A  dir/
 A  f1
 A  f2
 A  f3
 A  f4
+!  f5
 """
 
         repo = wt.branch.repository

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2010-09-13 02:32:44 +0000
+++ bzrlib/tests/test_errors.py	2010-10-11 05:06:26 +0000
@@ -672,6 +672,15 @@
         err = errors.NotBranchError('path', bzrdir=bzrdir)
         self.assertEqual('Not a branch: "path".', str(err))
 
+    def test_not_branch_bzrdir_with_recursive_not_branch_error(self):
+        class FakeBzrDir(object):
+            def open_repository(self):
+                # str() on the NotBranchError will trigger a call to this,
+                # which in turn will another, identical NotBranchError.
+                raise errors.NotBranchError('path', bzrdir=FakeBzrDir())
+        err = errors.NotBranchError('path', bzrdir=FakeBzrDir())
+        self.assertEqual('Not a branch: "path".', str(err))
+
     def test_not_branch_laziness(self):
         real_bzrdir = self.make_bzrdir('path')
         class FakeBzrDir(object):

=== modified file 'bzrlib/tests/test_script.py'
--- bzrlib/tests/test_script.py	2010-10-08 06:34:10 +0000
+++ bzrlib/tests/test_script.py	2010-10-27 23:27:41 +0000
@@ -161,7 +161,18 @@
 class TestExecution(script.TestCaseWithTransportAndScript):
 
     def test_unknown_command(self):
-        self.assertRaises(SyntaxError, self.run_script, 'foo')
+        """A clear error is reported for commands that aren't recognised
+
+        Testing the attributes of the SyntaxError instance is equivalent to
+        using traceback.format_exception_only and comparing with:
+          File "<string>", line 1
+            foo --frob
+            ^
+        SyntaxError: Command not found "foo"
+        """
+        e = self.assertRaises(SyntaxError, self.run_script, "$ foo --frob")
+        self.assertContainsRe(e.msg, "not found.*foo")
+        self.assertEquals(e.text, "foo --frob")
 
     def test_blank_output_mismatches_output(self):
         """If you give output, the output must actually be blank.

=== modified file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tests/test_setup.py	2010-11-05 14:07:45 +0000
@@ -43,30 +43,34 @@
         This tests that the build process and man generator run correctly.
         It also can catch new subdirectories that weren't added to setup.py.
         """
-        if not os.path.isfile('setup.py'):
-            raise TestSkipped('There is no setup.py file in current directory')
+        # setup.py must be run from the root source directory, but the tests
+        # are not necessarily invoked from there
+        self.source_dir = os.path.dirname(os.path.dirname(bzrlib.__file__))
+        if not os.path.isfile(os.path.join(self.source_dir, 'setup.py')):
+            raise TestSkipped(
+                'There is no setup.py file adjacent to the bzrlib directory')
         try:
             import distutils.sysconfig
             makefile_path = distutils.sysconfig.get_makefile_filename()
             if not os.path.exists(makefile_path):
-                raise TestSkipped('You must have the python Makefile installed to run this test.'
-                                  ' Usually this can be found by installing "python-dev"')
+                raise TestSkipped(
+                    'You must have the python Makefile installed to run this'
+                    ' test. Usually this can be found by installing'
+                    ' "python-dev"')
         except ImportError:
-            raise TestSkipped('You must have distutils installed to run this test.'
-                              ' Usually this can be found by installing "python-dev"')
+            raise TestSkipped(
+                'You must have distutils installed to run this test.'
+                ' Usually this can be found by installing "python-dev"')
         self.log('test_build running in %s' % os.getcwd())
-        install_dir = osutils.mkdtemp()
-        # setup.py must be run from the root source directory, but the tests
-        # are not necessarily invoked from there
-        self.source_dir = os.path.dirname(os.path.dirname(bzrlib.__file__))
+        root_dir = osutils.mkdtemp()
         try:
             self.run_setup(['clean'])
             # build is implied by install
             ## self.run_setup(['build'])
-            self.run_setup(['install', '--prefix', install_dir])
+            self.run_setup(['install', '--root', root_dir])
             self.run_setup(['clean'])
         finally:
-            osutils.rmtree(install_dir)
+            osutils.rmtree(root_dir)
 
     def run_setup(self, args):
         args = [sys.executable, './setup.py', ] + args

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2010-10-15 09:53:27 +0000
+++ bzrlib/workingtree.py	2010-11-04 17:48:47 +0000
@@ -353,8 +353,8 @@
         return control.open_workingtree(), relpath
 
     @staticmethod
-    def open_containing_paths(file_list, default_directory='.',
-        canonicalize=True, apply_view=True):
+    def open_containing_paths(file_list, default_directory=None,
+                              canonicalize=True, apply_view=True):
         """Open the WorkingTree that contains a set of paths.
 
         Fail if the paths given are not all in a single tree.
@@ -362,6 +362,8 @@
         This is used for the many command-line interfaces that take a list of
         any number of files and that require they all be in the same tree.
         """
+        if default_directory is None:
+            default_directory = u'.'
         # recommended replacement for builtins.internal_tree_files
         if file_list is None or len(file_list) == 0:
             tree = WorkingTree.open_containing(default_directory)[0]
@@ -375,9 +377,15 @@
                     view_str = views.view_display_str(view_files)
                     note("Ignoring files outside view. View is %s" % view_str)
             return tree, file_list
-        tree = WorkingTree.open_containing(file_list[0])[0]
+        if default_directory == u'.':
+            seed = file_list[0]
+        else:
+            seed = default_directory
+            file_list = [osutils.pathjoin(default_directory, f)
+                         for f in file_list]
+        tree = WorkingTree.open_containing(seed)[0]
         return tree, tree.safe_relpath_files(file_list, canonicalize,
-            apply_view=apply_view)
+                                             apply_view=apply_view)
 
     def safe_relpath_files(self, file_list, canonicalize=True, apply_view=True):
         """Convert file_list into a list of relpaths in tree.

=== modified file 'doc/en/release-notes/bzr-2.3.txt'
--- doc/en/release-notes/bzr-2.3.txt	2010-10-23 08:27:11 +0000
+++ doc/en/release-notes/bzr-2.3.txt	2010-11-05 15:57:40 +0000
@@ -5,10 +5,63 @@
 .. toctree::
    :maxdepth: 1
 
+bzr 2.3b4
+#########
+
+:Codename: Nirvana
+:2.3.b4: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+
 bzr 2.3b3
 #########
 
-:2.3b3: NOT RELEASED YET
+:2.3b3: 2010-11-05
 
 External Compatibility Breaks
 *****************************
@@ -38,9 +91,6 @@
   default format for new branches created with ``bzr init``.
   (Neil Martinsen-Burrell, #484101)
 
-Improvements
-************
-
 Bug Fixes
 *********
 
@@ -49,8 +99,17 @@
 * ``bzr status -r X..Y`` was failing because RevisionTree didn't implement
   ``get_shelf_manager``. (John Arbash Meinel, #662053)
 
+* Correctly add directory contents when the name was previously added as a
+  normal file, rather than throwing ``AttributeError: children`` during
+  smart_add. (Martin [gz], #251864)
+
+* Correctly handle the ``--directory`` option for all code paths of
+  ``resolve`` and ``shelve``, this was previously ignored when paths were
+  provided as parameters. When both are provided, ``--directory`` becomes
+  the base directory for the other paths. (Vincent Ladeuil, #670851)
+
 * Correctly set the Content-Type  header when http POSTing to comply
-  with stricter web frameworks. (Vincent Ladeuil, #655100)
+  with stricter web frameworks. (Vincent Ladeuil, #665100)
 
 * Don't force openssh to use protocol=2, since that is now the default.
   (Neil Martinsen-Burrell, #561061)
@@ -61,6 +120,16 @@
 
 * Make ``bzr tag --quiet`` really quiet. (Neil Martinsen-Burrell, #239523)
 
+* Missing files (files bzr add'ed and then OS deleted) are now shown in ``bzr
+  status`` output. (Rory Yorke, #134168)
+
+* ``NotBranchError`` no longer allows errors from calling
+  ``bzrdir.open_repository()`` to propagate.  This is unhelpful at best,
+  and at worst can trigger infinite loops in callers.  (Andrew Bennetts)
+  
+* Windows installers no longer requires the Microsoft vcredist to be
+  installed. (Martin [gz], Gary van der Merwe, #632465)
+
 Documentation
 *************
 
@@ -72,7 +141,7 @@
 
 * Document the comment character in the .bzrignore file, including a
   workaround for ignore patterns that begin with #.
-  (Neil Martinsen-Burrell, #621515)
+  (Neil Martinsen-Burrell, #631515)
 
 API Changes
 ***********
@@ -109,6 +178,8 @@
   get rid of some 'bytes left on the HTTP socket' useless log messages.
   (Vincent Ladeuil, #655557)
 
+* ``bzrlib.tests.per_workingtree.TestCaseWithWorkingTree.make_branch_builder``
+  respects its ``relpath`` parameter. (Vincent Ladeuil)
 
 bzr 2.3b2
 #########

=== modified file 'doc/en/whats-new/whats-new-in-2.2.txt'
--- doc/en/whats-new/whats-new-in-2.2.txt	2010-10-06 16:27:25 +0000
+++ doc/en/whats-new/whats-new-in-2.2.txt	2010-11-04 13:46:02 +0000
@@ -23,7 +23,7 @@
 :doc:`../release-notes/index` for a full list.
 
 Bazaar 2.2.1 includes all the fixes from 2.1.3 and 2.0.6 (that
-weren't included in 2.20).
+weren't included in 2.2.0).
 
 See the :doc:`../release-notes/index` for details.
 

=== modified file 'doc/en/whats-new/whats-new-in-2.3.txt'
--- doc/en/whats-new/whats-new-in-2.3.txt	2010-10-15 15:05:09 +0000
+++ doc/en/whats-new/whats-new-in-2.3.txt	2010-11-05 15:19:01 +0000
@@ -121,7 +121,7 @@
 Configuration
 *************
 
-``bzr`` can be configure via environment variables, command-line options
+``bzr`` can be configured via environment variables, command-line options
 and configurations files. We've started working on unifying this and give
 access to more options. The first step is a new ``bzr config`` command that
 can be used to display the active configuration options in the current

=== modified file 'setup.py'
--- setup.py	2010-10-14 21:22:48 +0000
+++ setup.py	2010-11-05 14:07:45 +0000
@@ -689,7 +689,10 @@
 
     # MSWSOCK.dll is a system-specific library, which py2exe accidentally pulls
     # in on Vista.
-    dll_excludes.extend(["MSWSOCK.dll", "MSVCP60.dll", "powrprof.dll"])
+    dll_excludes.extend(["MSWSOCK.dll",
+                         "MSVCP60.dll",
+                         "MSVCP90.dll",
+                         "powrprof.dll"])
     options_list = {"py2exe": {"packages": packages + list(additional_packages),
                                "includes": includes,
                                "excludes": excludes,

